"""
This does the full thing:  it views from the retina, through V1,
to pyramid detection, then sets a fact on.  It also has the planning in.
So, it does the commands that are activated.  This can be set in the
short version in runNoCallBack, or all can be tested with the RunCallBack
version.  I haven't done the full range of error checking for planning.
It runs on a 4 chip spinnaker board with 3-1 neurons.
It runs on both nest and the spinnaker chip.  Set this in nealParams.py.
"""

#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO

#imports for image reading
import numpy
from PIL import Image

#NEAL files and functions
from nealParams import *
from visionClass import visionArea
from maesSpinnV8 import *
from parseClass import parseArea
from controlClass import controlArea

def nealProjection(preNeurons,postNeurons, connectorList,inhExc):
    if simulator_name == 'spiNNaker':
        Projection(preNeurons, postNeurons, connectorList,target=inhExc)
    elif simulator_name == 'nest':
        Projection(preNeurons, postNeurons, connectorList)
    else:
        print 'bad simulator for nealProjection'

def runNoCallBack():
    run(SIM_LENGTH)

def connectVisionToPlan():
    print "cvp"

def connectParseResultToGoal(parseCells,goalCells,parseNumber,goalNumber):
    connectors = []
    wt = 10.0
    for fromNeuron in range (0,5):
        inpNeuron = parseNumber*5 + fromNeuron
        for toNeuron in range (0,5):
            outNeuron = goalNumber*5 + toNeuron
            connectors = connectors + [(inpNeuron,outNeuron,wt,DELAY)]
    excConnector = FromListConnector(connectors)
    nealProjection(parseCells, goalCells, excConnector,'excitatory')

def connectParseToPlan():
    stateUnits = parse.getStates()
    goalUnits = maes1.getGoalCells()
    connectParseResultToGoal(stateUnits,goalUnits,3,1) #turn left
    connectParseResultToGoal(stateUnits,goalUnits,5,0) #turn right
    connectParseResultToGoal(stateUnits,goalUnits,14,2) #move forward
    connectParseResultToGoal(stateUnits,goalUnits,16,3) #move back
    connectParseResultToGoal(stateUnits,goalUnits,18,5) #move left
    connectParseResultToGoal(stateUnits,goalUnits,20,4) #move right
    connectParseResultToGoal(stateUnits,goalUnits,9,6) #turn toward the pyramid
    connectParseResultToGoal(stateUnits,goalUnits,11,7) #turn toward the stalac

def oneParseMovesOneControl(parseCells,controlCells,parseCA,newControlCA):
    connectors = []
    wt = 0.03
    for fromNeuron in range (0,5):
        inpNeuron = parseCA*5 + fromNeuron
        for toNeuron in range (0,5):
            outNeuron = newControlCA*5 + toNeuron
            connectors = connectors + [(inpNeuron,outNeuron,wt,DELAY)]
    excConnector = FromListConnector(connectors)
    nealProjection(parseCells, controlCells, excConnector,'excitatory')


def oneControlStopsOneParse(parseCells,controlCells,parseCA,controlCA):
    connectors = []
    wt = -25.0
    for fromNeuron in range (0,5):
        inpNeuron = controlCA*5 + fromNeuron
        for toNeuron in range (0,5):
            outNeuron = parseCA*5 + toNeuron
            connectors = connectors + [(inpNeuron,outNeuron,wt,DELAY)]
    inhConnector = FromListConnector(connectors)
    nealProjection(controlCells, parseCells, inhConnector,'inhibitory')

def oneControlStartsOneParse(parseCells,controlCells,parseCA,controlCA):
    connectors = []
    wt = 2.0
    for fromNeuron in range (0,5):
        inpNeuron = controlCA*5 + fromNeuron
        for toNeuron in range (0,5):
            outNeuron = parseCA*5 + toNeuron
            connectors = connectors + [(inpNeuron,outNeuron,wt,DELAY)]
    inhConnector = FromListConnector(connectors)
    nealProjection(controlCells, parseCells, inhConnector,'excitatory')

def connectParseAndControl():
    #consider need to start first parse state to parse again.
    parseUnits = parse.getStates()
    controlUnits = control.getStates()
    for parseState in (3,5,9,11,14,16,18,20): #final parse states
        oneParseMovesOneControl(parseUnits,controlUnits,parseState,1)
        oneControlStopsOneParse(parseUnits,controlUnits,parseState,1)
    oneControlStartsOneParse(parseUnits,controlUnits,0,2)


#------------Main Body---------------
#import the simulator via nealParams.py
#simulator_name = get_script_args(1)[0]  
#exec("from pyNN.%s import *" % simulator_name)

setup(timestep=DELAY,min_delay=DELAY,max_delay=DELAY,debug=0)


#----------------create neurons
parse = parseArea(simulator_name)
#--------------external input parse needs to be before connections 
parse.setSpikeServerInputs()
parse.allocateParseNeurons()

vision = visionArea(simulator_name)
vision.setWhatsOn(2) #2,1 off
vision.allocateVisionNeurons()

maes1 = Maes()

control = controlArea()
control.allocateControlNeurons()

#--------------external input parse needs to be before connections 
#parse.setInputs()

spikes = [[7]]
controlInputSource=Population(1,SpikeSourceArray,{'spike_times':spikes})

#---------setup connections
vision.makeVisionSynapses()

maes1.network()
maes1.networkFacts()
maes1.networkError()

parse.setConnections()

control.makeControlSynapses(controlInputSource)

#connect subsytems
connectVisionToPlan()

#Connect Vision Facts to Plan Facts
visionFactRightPyramid = vision.getRightPyramid()
visionFactLeftPyramid = vision.getLeftPyramid()
visionFactRightStalactite = vision.getRightStalactite()
visionFactLeftStalactite = vision.getLeftStalactite()
maesFacts = maes1.getFactPop()

rightConnectors = []
leftConnectors = []
pyrConnectors = []
stalConnectors = []
wt = 4.0
for inpNeuron in range (0,5):
    for toNeuron in range (0,5):
        rightConnectors = rightConnectors + [(inpNeuron,toNeuron,wt,DELAY)]
        leftConnectors = leftConnectors + [(inpNeuron,toNeuron+5,wt,DELAY)]
        pyrConnectors = pyrConnectors + [(inpNeuron,toNeuron+10,wt,DELAY)]
        stalConnectors = stalConnectors + [(inpNeuron,toNeuron+15,wt,DELAY)]
rightExcConnector = FromListConnector(rightConnectors)
leftExcConnector = FromListConnector(leftConnectors)
pyrExcConnector = FromListConnector(pyrConnectors)
stalExcConnector = FromListConnector(stalConnectors)
nealProjection(visionFactRightPyramid, maesFacts, rightExcConnector,'excitatory')
nealProjection(visionFactRightPyramid, maesFacts, pyrExcConnector,'excitatory')
nealProjection(visionFactLeftPyramid, maesFacts, leftExcConnector,'excitatory')
nealProjection(visionFactLeftPyramid, maesFacts, pyrExcConnector,'excitatory')
nealProjection(visionFactRightStalactite, maesFacts, rightExcConnector,'excitatory')
nealProjection(visionFactRightStalactite, maesFacts, stalExcConnector,'excitatory')
nealProjection(visionFactLeftStalactite, maesFacts, leftExcConnector,'excitatory')
nealProjection(visionFactLeftStalactite, maesFacts, stalExcConnector,'excitatory')

connectParseToPlan()
connectParseAndControl()


#-------------------setup recording
vision.setVisionRecording()
maes1.setPlanRecording()
parse.setRecording()
#control.setControlRecording()

runNoCallBack()

#--------------print results
print "done running"

#vision.printVisionNets()
#vision.printObjects()
#maes1.printMaes() 
#parse.printResults()
#control.printControlNets()


end()

