"""
The Control Class.
"""


#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO

from nealParams import *


class controlArea:
    NUMBER_STATES = 3

    def nealProjection(self,preNeurons,postNeurons, connectorList,inhExc):
        if simulator_name == 'spiNNaker':
            Projection(preNeurons, postNeurons, connectorList,target=inhExc)
        elif simulator_name == 'nest':
            Projection(preNeurons, postNeurons, connectorList)
        else:
            print 'bad simulator for nealProjection'

    #---create neurons
    def allocateControlNeurons(self):
        conCellPars = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.0}  
        self.StateCells=Population(self.NUMBER_STATES*5,IF_curr_exp,conCellPars)



    def makeCASynapses(self,CA):
        synWeight = 25.0
        connector = []
        for fromOff in range (0,5):
            fromNeuron = CA*5+fromOff
            for toOff in range (0,5):
                if (fromOff != toOff):
                    toNeuron = CA*5+toOff
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.StateCells,self.StateCells,excConnector,
                            'excitatory')

    def connectAdjacentStates(self,fromState,toState):
        #synWeight = 0.5 #moves forward for test
        #synWeight = 0.2 #standalone
        synWeight = 0.02
        inhWeight = -5.0
        connector = []
        inhConnector = []
        for fromOff in range (0,5):
            fromNeuron = fromState*5+fromOff
            for toOff in range (0,5):
                toNeuron = toState*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
                inhConnector=inhConnector+[(toNeuron,fromNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.StateCells,self.StateCells,excConnector,
                            'excitatory')
        inhConnectorList = FromListConnector(inhConnector)
        self.nealProjection(self.StateCells,self.StateCells,inhConnectorList,
                            'inhibitory')

    def connectAdjacentStatesSlide(self,fromState,toState):
        #synWeight = 0.5 #moves forward for test
        #synWeight = 0.2 #standalone
        synWeight = 0.2
        inhWeight = -5.0
        connector = []
        inhConnector = []
        for fromOff in range (0,5):
            fromNeuron = fromState*5+fromOff
            for toOff in range (0,5):
                toNeuron = toState*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
                inhConnector=inhConnector+[(toNeuron,fromNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.StateCells,self.StateCells,excConnector,
                            'excitatory')
        inhConnectorList = FromListConnector(inhConnector)
        self.nealProjection(self.StateCells,self.StateCells,inhConnectorList,
                            'inhibitory')

    def connectInputToStart(self,inputCell):
        synWeight = 25.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron,synWeight,DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(inputCell,self.StateCells,excConnector,
                            'excitatory')

    def makeControlSynapses(self,inputCell):
        for cCA in range (0,self.NUMBER_STATES):
            self.makeCASynapses(cCA)
        self.connectInputToStart(inputCell)
        self.connectAdjacentStates(0,1)
        self.connectAdjacentStatesSlide(1,2)
        self.connectAdjacentStatesSlide(2,0)


    def setControlRecording(self):
        if simulator_name == 'spiNNaker':
            self.StateCells.record()
        elif simulator_name == 'nest':
            self.StateCells.record('spikes')


    def nestPrintSpikes(self, pop):
		outDat = pop.get_data()
		for seg in outDat.segments:
			print("\n")
			print(seg.spiketrains)
			
    def printControlNets(self):
        if simulator_name == 'spiNNaker':
            self.StateCells.printSpikes('results/controlStates.sp')
        elif simulator_name == 'nest':
            self.nestPrintSpikes(self.StateCells)

    def getStates(self):
        return self.StateCells
        




