"""
"""
#from pyNN import *
#from pyNN.models import *
# cell assembly class for simple oscillator
#class Oscillator(pyNN.common):

from nealParams import *


goalsList 	= 	("TR","TL", "FW", "BW", "MR1", "ML1", "TP", "TS", "TO")
goalsNPList	=	("MR2","ML2")
modulesList 	= 	("TR","TL", "FW", "BW", "RO", "LO", "MERR1", "MERR2", "MERR20")
actionsList 	= 	("TR", "TL", "FW", "BW", "AERR1", "AERR2", "AERR20")
factsList 	=	("RIGHT", "LEFT", "PYR", "STAL", "CENTRE")


MAX_GOALS=len(goalsList)				#maximum number of Goals
MAX_GOALS_NP=len(goalsNPList)				#maximum number of non-primed Goals, for compound statements
MAX_FACTS=len(factsList)				#maximum number of facts
MAX_ACTIONS=len(actionsList)				#maximum number of actions, including errors
MAX_MODULES=len(modulesList)				#maximum number of modules, including errors
SUB_POPULATION_SIZE=5					#size of sub population
PULSE_DURATION=10					#for DC injection the duration of pulse in milli
DELAY = 1.0						#delay in milliseconds


		
def NealProjection(preNeurons,postNeurons, connectorList,inhExc):
        if simulator_name == 'spiNNaker':
            Projection(preNeurons, postNeurons, connectorList,target=inhExc)
        elif simulator_name == 'nest':
            Projection(preNeurons, postNeurons, connectorList)
        else:
            print 'bad simulator for nealProjection'


class Oscillator:
	cell_params = {'tau_refrac' : 5.0, 'v_rest' : -65.0,
                    'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
                    'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                    'i_offset' : 0.0, 'cm': 0.1}

	
        #internal weights between neurons in same cell assembly
	if simulator_name == 'spiNNaker':
		int_weight=10.0			
	elif simulator_name == 'nest':
		int_weight = 2.00 		
	else:
		print 'bad simulator for oscillator dec'


	inh_weight=2.50			#afferent weights from other cell assemblies - inhibitory
	exc_weight=2.500 		
	#constructor
	def __init__(self, label, maxSubPop, coeff):
		self.setName(label)
		self.int_weight*=coeff
		if(label=='ACTIONS'):
			self.size= 1 * maxSubPop #actions have only population of size1
			self.subpop=maxSubPop
			self.setPop()
			#no internal connections for actions
		else:
			self.size = SUB_POPULATION_SIZE*maxSubPop
			self.subpop = maxSubPop
			self.setPop()
			self.sub_pop_connect(self.getPop(),self.getPop(), self.int_weight,'excitatory')

	#get
	def getSize(self):
		return self.size
	def getPop(self):
		return self.pop
	def getSubPop(self):
		return self.subpop
	def getName(self):
		return self.name
	def getIntWeight(self):
		return self.int_weight
	def getInhWeight(self):
		return self.inh_weight
	def getExcWeight(self):
		return self.exc_weight
	#set weight
	def setSize(self,x):
		self.size=x
	def setName(self,x):
		self.name=x
	def setSubPop(self,x):
		self.subpop = x
	def setIntWeight(self, w):
		self.int_weight=w
	def setInhWeight(self, w):
		self.inh_weight = w
	def setExcWeight(self, w):
		self.exc_weight=w
	def setName(self, n):
		self.name = n
	#print
	def printOsc(self):
#		print parameters
		print "\n name in Oscillator %s"%self.getName()
		print "\n size in Oscillator %d"%self.getSize()
		print "\n max sub pop  in Oscillator: %d"%self.getSubPop()
	
	#set connections
	def sub_pop_connect(self, fromCells,toCells,weight,inhOrExcite):
		connectors = []
		for spi in range(0,self.getSubPop()):	
			LBspi = spi * SUB_POPULATION_SIZE
			for inpNeuron in range (LBspi, LBspi+SUB_POPULATION_SIZE):
				for toNeuron in range ( LBspi,LBspi+SUB_POPULATION_SIZE):
					if (inpNeuron!=toNeuron):
						connectors = connectors+[(inpNeuron,toNeuron,weight,DELAY)]
		excConnector = FromListConnector(connectors)
		#Projection(fromCells, toCells, excConnector)
		NealProjection(fromCells, toCells, excConnector, inhOrExcite)
	
	#set population
	def setPop(self):
		self.pop = Population(self.getSize(),IF_curr_exp,self.cell_params)
	def recordSpikes(self):
		self.getPop().record()
		#self.getPop().record('spikes')


	def injection(self,start_time,t):
		t*=SUB_POPULATION_SIZE
		pulse=DCSource(amplitude=3.55, start=start_time, stop=start_time+PULSE_DURATION)
		pulse.inject_into(Assembly(self.getPop()[t,t+1,t+2,t+3,t+4]))
	def switchOff(self, start_time,t):
		t*=SUB_POPULATION_SIZE
		pulse=DCSource(amplitude=-5.00, start=start_time, stop=start_time+PULSE_DURATION)
		pulse.inject_into(Assembly(self.getPop()[t,t+1,t+2,t+3,t+4]))
		
class Maes:

	def __init__(self):
		injectionInterval=100
		currentTime=0
		#Create the five populations
		self.Facts = Oscillator('FACTS', MAX_FACTS,0.8)
		self.Goals = Oscillator("GOALS", MAX_GOALS,1.0)
		self.GoalsNP = Oscillator("GOALSnp", MAX_GOALS_NP,1.0)
		self.Actions = Oscillator("ACTIONS", MAX_ACTIONS,1.0)
		self.Modules = Oscillator("MODULES", MAX_MODULES,0.9)
				

	#get 
	def getModules(self):
		return self.Modules
	def getActions(self):
		return self.Actions
	def getGoals(self):
		return self.Goals
	def getGoalsNP(self):
		return self.GoalsNP
	def getFacts(self):
		return self.Facts
	def getSubPopIndex(self,spi):  #SPI = sub population index
		return spi*POPULATION_SIZE 
	def getModuleIndex(self, x):
		return modulesList.index(x)
	def getActionIndex(self, x):
		return actionsList.index(x)
	def getFactIndex(self,x):
		return factsList.index(x)
	def getGoalIndex(self,x):
		return goalsList.index(x)
	def getGoalNPIndex(self,x):
		return goalsNPList.index(x)
	#set up connections
	def connectionSubPop(self, pop1, pop2, sp1, sp2, coeff):
		if (pop1.getName()=='ACTIONS'):
			self.connectionSubPopActionOut(pop1,pop2,sp1,sp2,coeff)
		elif (pop2.getName()=='ACTIONS'):
			self.connectionSubPopActionIn(pop1,pop2,sp1,sp2,coeff)
		else:
			self.connectionSubPopNoAction(pop1,pop2,sp1,sp2,coeff)
		
	def connectionSubPopActionOut(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp2 = sp2*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for l in range(LBsp2, LBsp2+SUB_POPULATION_SIZE):
			cons=cons+[(sp1,l,w,DELAY)]
		conList=FromListConnector(cons)
		NealProjection(pop1.getPop(), pop2.getPop(), conList, inhOrExcite)

	def connectionSubPopActionIn(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp1 = sp1*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for l in range(LBsp1, LBsp1+SUB_POPULATION_SIZE):
			cons=cons+[(l,sp2, w,DELAY)]
		conList=FromListConnector(cons)
		NealProjection(pop1.getPop(), pop2.getPop(), conList, inhOrExcite)



	def connectionSubPopNoAction(self, pop1, pop2, sp1, sp2, coeff):
		inhOrExcite='excitatory'
		cons=[]
		w = coeff*pop1.getExcWeight()
		LBsp1 = sp1*SUB_POPULATION_SIZE #Lower Bound Index for sub population 1
		LBsp2 = sp2*SUB_POPULATION_SIZE #Lower Bound Index for sub population 2
		if (coeff<0):
			inhOrExcite='inhibitory'
			w = coeff*pop1.getInhWeight()
		#AllToAll Connection
		for k in range(LBsp1, LBsp1+SUB_POPULATION_SIZE):
			for l in range(LBsp2, LBsp2+SUB_POPULATION_SIZE):
				cons=cons+[(k,l,w,DELAY)]
		conList=FromListConnector(cons)
		NealProjection(pop1.getPop(), pop2.getPop(), conList, inhOrExcite)

	def injectionTestCRH(self, inputSource,goalNum):
		#connect input Source to goal like i
		cons = []
		for k in range(0,5):
			toNeuron = k + (goalNum*5)
			cons=cons+[(0,toNeuron,3.5,DELAY)]
		conList=FromListConnector(cons)
		NealProjection(inputSource,self.Goals.getPop(), conList,'excitatory')


	def injectionTest(self, interval):
		#simple test
		self.Goals.injection(interval,self.getGoalIndex("TR"))
		self.Goals.injection(interval+100, self.getGoalIndex("TL"))
		self.Goals.injection(interval+200, self.getGoalIndex("FW"))
		self.Goals.injection(interval+300, self.getGoalIndex("BW"))
		#compound test
		self.Goals.injection(interval+400, self.getGoalIndex("MR1"))
		self.Goals.injection(interval+500, self.getGoalIndex("ML1"))
		
	def injectionTest2(self, interval):
		#turn right
		self.Facts.injection(interval, self.getFactIndex("PYR"))
		self.Facts.injection(interval, self.getFactIndex("RIGHT"))
		self.Goals.injection(interval, self.getGoalIndex("TP"))
		self.Facts.switchOff(interval+90, self.getFactIndex("PYR"))
		self.Facts.switchOff(interval+90, self.getFactIndex("RIGHT"))
		#turn left
		self.Facts.injection(interval+100, self.getFactIndex("STAL") )
		self.Facts.injection(interval+100, self.getFactIndex("LEFT") )
		self.Goals.injection(interval+100, self.getGoalIndex("TS") )
		self.Facts.switchOff(interval+190, self.getFactIndex("STAL") )
		self.Facts.switchOff(interval+190, self.getFactIndex("LEFT") )
		#turn right
		self.Facts.injection(interval+300, self.getFactIndex("STAL") )
		self.Facts.injection(interval+300, self.getFactIndex("RIGHT") )
		self.Goals.injection(interval+300, self.getGoalIndex("TS") )
		self.Facts.switchOff(interval+390, self.getFactIndex("STAL") )
		self.Facts.switchOff(interval+390, self.getFactIndex("RIGHT") )
		#turn left
		self.Facts.injection(interval+200, self.getFactIndex("PYR") )
		self.Facts.injection(interval+200, self.getFactIndex("LEFT") )
		self.Goals.injection(interval+200, self.getGoalIndex("TP") )
		self.Facts.switchOff(interval+290, self.getFactIndex("PYR") )
		self.Facts.switchOff(interval+290, self.getFactIndex("LEFT") )
		#move towards object - requires centre fact and goal (and object)
		self.Facts.injection(interval+400, self.getFactIndex("CENTRE"))
		self.Goals.injection(interval+400, self.getGoalIndex("TO"))
		self.Facts.switchOff(interval+490, self.getFactIndex("CENTRE"))
		
	def injectionTest3(self, interval):
		#error no fact object pyramid err1
		self.Facts.injection(interval, self.getFactIndex("RIGHT"))
		self.Goals.injection(interval, self.getGoalIndex("TP"))
		self.Facts.switchOff(interval+90, self.getFactIndex("RIGHT"))
		#error no fact object stalactite err2
		self.Facts.injection(interval+100, self.getFactIndex("RIGHT"))
		self.Goals.injection(interval+100, self.getGoalIndex("TS"))
		self.Facts.switchOff(interval+190, self.getFactIndex("RIGHT"))
		#error no facts, no direction, ERR20
		self.Facts.injection(interval+200, self.getFactIndex("PYR"))
		self.Goals.injection(interval+200, self.getGoalIndex("TP"))
		self.Facts.switchOff(interval+290, self.getFactIndex("PYR"))
		
	def network(self):
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("TR"),self.getModuleIndex("TR"), 0.055)
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("TL"),self.getModuleIndex("TL"), 0.055)
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("FW"),self.getModuleIndex("FW"), 0.055)
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("BW"),self.getModuleIndex("BW"), 0.055)
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("MR1"),self.getModuleIndex("TR"), 0.075)
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("ML1"),self.getModuleIndex("TL"), 0.075)
		#center fact required for TO - towards object - to work. Probably need object??
		#also need to inhibit this once action has reached conclusion
		self.connectionSubPop(self.Goals,self.Modules,self.getGoalIndex("TO"), self.getModuleIndex("FW"), 0.0275) 
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("FW"), self.getGoalIndex("TO"), -1.0)
		
		self.connectionSubPop(self.Modules,self.Actions,self.getModuleIndex("TR"),self.getActionIndex("TR"), 0.055)
		self.connectionSubPop(self.Modules,self.Actions,self.getModuleIndex("TL"),self.getActionIndex("TL"), 0.055)
		self.connectionSubPop(self.Modules,self.Actions,self.getModuleIndex("FW"),self.getActionIndex("FW"), 0.055)
		self.connectionSubPop(self.Modules,self.Actions,self.getModuleIndex("BW"),self.getActionIndex("BW"), 0.055)

		self.connectionSubPop(self.GoalsNP,self.Modules,self.getGoalNPIndex("MR2"),self.getModuleIndex("FW"), 0.075)
		self.connectionSubPop(self.GoalsNP,self.Modules,self.getGoalNPIndex("ML2"),self.getModuleIndex("FW"), 0.075)

		
		self.connectionSubPop(self.Goals, self.GoalsNP, self.getGoalIndex("MR1"), self.getGoalNPIndex("MR2"), 0.03975 )
		self.connectionSubPop(self.Goals, self.GoalsNP, self.getGoalIndex("ML1"), self.getGoalNPIndex("ML2"), 0.03975 )		

		self.connectionSubPop(self.Actions, self.GoalsNP, self.getActionIndex("TR"), self.getGoalNPIndex("MR2"),0.035)
		self.connectionSubPop(self.Actions, self.GoalsNP, self.getActionIndex("TL"), self.getGoalNPIndex("ML2"),0.035)
		
		self.connectionSubPop(self.Actions,self.Goals, self.getActionIndex("TR"), self.getGoalIndex("TR"), -1.0)
		self.connectionSubPop(self.Actions,self.Goals, self.getActionIndex("TL"), self.getGoalIndex("TL"), -1.0)
		self.connectionSubPop(self.Actions,self.Goals, self.getActionIndex("FW"), self.getGoalIndex("FW"), -1.0)
		self.connectionSubPop(self.Actions,self.Goals, self.getActionIndex("BW"), self.getGoalIndex("BW"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("TR"), self.getModuleIndex("TR"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("TL"), self.getModuleIndex("TL"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("FW"), self.getModuleIndex("FW"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("BW"), self.getModuleIndex("BW"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TR"), self.getGoalIndex("MR1"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TL"), self.getGoalIndex("ML1"), -1.0)

		self.connectionSubPop(self.Actions,self.GoalsNP, self.getActionIndex("FW"), self.getGoalNPIndex("MR2"), -1.0)
		self.connectionSubPop(self.Actions,self.GoalsNP, self.getActionIndex("FW"), self.getGoalNPIndex("ML2"), -1.0)

		
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TP"), self.getModuleIndex("RO"), 0.0125)
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TP"), self.getModuleIndex("LO"), 0.0125)
		
		self.connectionSubPop(self.Goals, self.Modules, self.getGoalIndex("TS"), self.getModuleIndex("LO"), 0.0125)
		self.connectionSubPop(self.Goals, self.Modules, self.getGoalIndex("TS"), self.getModuleIndex("RO"), 0.0125)
		
		self.connectionSubPop(self.Modules, self.Actions, self.getModuleIndex("LO"), self.getActionIndex("TL"), 0.0575)
		self.connectionSubPop(self.Modules, self.Actions, self.getModuleIndex("RO"), self.getActionIndex("TR"), 0.0575)

		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TR"), self.getGoalIndex("TP"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TL"), self.getGoalIndex("TP"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TR"), self.getGoalIndex("TS"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("TL"), self.getGoalIndex("TS"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("TR"), self.getModuleIndex("RO"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("TL"), self.getModuleIndex("LO"), -1.0)
		
	def networkFacts(self):
		if simulator_name == 'spiNNaker':
			#dirFactPrimeMod = 0.020 
			dirFactPrimeMod = 0.010 
		elif simulator_name == 'nest':
			#dirFactPrimeMod = 0.025
			dirFactPrimeMod = 0.05
                else:
			print 'bad simulator for networkFacts'

		#facts and modules
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("RIGHT"), self.getModuleIndex("RO"), dirFactPrimeMod) 
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("LEFT"), self.getModuleIndex("LO"), dirFactPrimeMod)
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("PYR"), self.getModuleIndex("LO"), 0.0125)
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("PYR"), self.getModuleIndex("RO"), 0.0125)		
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("STAL"), self.getModuleIndex("LO"), 0.0125)
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("STAL"), self.getModuleIndex("RO"), 0.0125)		


		#self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("CENTRE"), self.getModuleIndex("FW"), 0.0275)
		
		
		#facts inhibit modules
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("RIGHT"), self.getModuleIndex("LO"), -1.75)
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("LEFT"), self.getModuleIndex("RO"), -1.75)
		
		
	def networkError(self):	
	#for generic goal turn towards object, that requires at least the fact object. The fact "left" or "right" will be for a different error.
		#errors for turn towards object, but no or wrong object present
		#err1 and err2
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TP"), self.getModuleIndex("MERR1"), 0.04)
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("PYR"), self.getModuleIndex("MERR2"), 0.0175)
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("STAL"), self.getModuleIndex("MERR1"), 0.0175)
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TS"), self.getModuleIndex("MERR2"), 0.04)
		
		#errors for object present but no direction
		#err20
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TP"), self.getModuleIndex("MERR20"), 0.04)
		self.connectionSubPop(self.Goals,self.Modules, self.getGoalIndex("TS"), self.getModuleIndex("MERR20"), 0.04)
		
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("STAL"), self.getModuleIndex("MERR20"), 0.0175)
		self.connectionSubPop(self.Facts,self.Modules, self.getFactIndex("PYR"), self.getModuleIndex("MERR20"), 0.0175)
		
		#network modules and actions - excitatory
		self.connectionSubPop(self.Modules,self.Actions, self.getModuleIndex("MERR1"), self.getActionIndex("AERR1"), 0.0575)
		self.connectionSubPop(self.Modules,self.Actions, self.getModuleIndex("MERR2"), self.getActionIndex("AERR2"), 0.0575)
		self.connectionSubPop(self.Modules,self.Actions, self.getModuleIndex("MERR20"), self.getActionIndex("AERR20"), 0.0575)

		#network facts and errors - inhibitory
 		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("LEFT"), self.getModuleIndex("MERR20"), -1.0)
 		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("RIGHT"), self.getModuleIndex("MERR20"), -1.0)
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("STAL"), self.getModuleIndex("MERR2"), -1.0)
		self.connectionSubPop(self.Facts, self.Modules, self.getFactIndex("PYR"), self.getModuleIndex("MERR1"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("AERR1"), self.getModuleIndex("MERR1"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("AERR2"), self.getModuleIndex("MERR2"), -1.0)
		self.connectionSubPop(self.Actions, self.Modules, self.getActionIndex("AERR20"), self.getModuleIndex("MERR20"), -1.0)
		
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("AERR1"), self.getGoalIndex("TP"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("AERR2"), self.getGoalIndex("TS"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("AERR20"), self.getGoalIndex("TS"), -1.0)
		self.connectionSubPop(self.Actions, self.Goals, self.getActionIndex("AERR20"), self.getGoalIndex("TP"), -1.0)
		
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR1"), self.getModuleIndex("TL"), -1.0)
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR1"), self.getModuleIndex("TR"), -1.0)
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR2"), self.getModuleIndex("TL"), -1.0)
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR2"), self.getModuleIndex("TR"), -1.0)
		
		
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR20"), self.getModuleIndex("TL"), -1.0)
		self.connectionSubPop(self.Modules, self.Modules, self.getModuleIndex("MERR20"), self.getModuleIndex("TR"), -1.0)
	

	def getFactPop(self):
		return self.getFacts().getPop()

	def setPlanRecording(self):
		if simulator_name == 'spiNNaker':
			self.getActions().recordSpikes()
			#self.getFacts().recordSpikes()
			#self.getGoals().recordSpikes()
			self.getModules().recordSpikes()
		elif simulator_name == 'nest':
			self.getActions().getPop().record('spikes') 
			self.getFacts().getPop().record('spikes')
			self.getGoals().getPop().record('spikes')
			self.getModules().getPop().record('spikes')


	def print_spikes(self,of, label, pop):
		print "\n\n%s\n" % label
		of.write("\n%s\n" % label)
		outDat = pop.get_data()
		for seg in outDat.segments:
			print("\n")
			print(seg.spiketrains)
			of.write(str(seg.spiketrains))
			

	
	def printMaes(self): 
		if simulator_name == 'spiNNaker': 
			self.getActions().getPop().printSpikes('results/actions.sp')
			self.getFacts().getPop().printSpikes('results/facts.sp')
			self.getGoals().getPop().printSpikes('results/goals.sp')
			self.getModules().getPop().printSpikes('results/modules.sp')
		elif simulator_name == 'nest':
			of = open('maes.dat', 'w')
			self.print_spikes(of, "goals", self.getGoals().getPop())
			self.print_spikes(of, "modules", self.getModules().getPop())
			self.print_spikes(of, "facts", self.getFacts().getPop())
			self.print_spikes(of, "actions", self.getActions().getPop())

	def getGoalCells(self): 
		return self.Goals.getPop()
